/* ExperimentUtils.java
 * This source file is part of the DDPref software suite that implements the
 * DDPref language for preferences over sets of objects, and algorithms
 * for learning such preferences automatically from training data.
 * 
 * Copyright (C) 2006  Eric Eaton
 * 
 * If you use or extend this software, please cite it as:
 *   Marie desJardins, Eric Eaton, and Kiri Wagstaff.  (2006).
 *   Learning User Preferences for Sets of Objects.
 *   In Proceedings of the 23rd International Conference
 *   on Machine Learning, June 25--29, Pittsburgh, PA.
 *  
 * This work was partly supported by NSF grant ITR #0325329 and was partly carried
 * out at the Jet Propulsion Laboratory, California Institute of Technology,
 * under a contract with the National Aeronautics and Space Administration.
 * 
 * Developed at the University of Maryland Baltimore County,
 *                  Dept. of Computer Science and Electrical Engineering,
 *                  MAPLE Laboratory (http://maple.cs.umbc.edu).
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

/**
 * @author Eric Eaton
 */

package ddpref.experiments;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.text.DecimalFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import weka.core.InstanceComparator;
import weka.core.Instance;
import weka.core.Instances;

import org.apache.commons.math.stat.descriptive.SummaryStatistics;

import ddpref.WekaUtils;

public class ExperimentUtils {
	
	
	/** Outputs a data table with column and row labels
	 * 
	 * @param rowLabels
	 * @param colLabels
	 * @param stats A matrix where the first indice is the row, the second is the column.
	 */
	public static void printMatrix(String[] rowLabels, String[] colLabels, SummaryStatistics[][] stats, DecimalFormat df){
		System.out.print("\t");
		for (int col = 0; col < colLabels.length; col++) {
			System.out.print(colLabels[col]);
			if (col != colLabels.length-1) {
				System.out.print("\t");
			}
		}
		System.out.println();
		for (int row = 0; row < stats.length; row++) {
			System.out.print(rowLabels[row]+"\t");
			for (int col = 0; col < stats[row].length; col++) {
				if (stats[row][col] == null) {
					System.out.print("-");
				} else {
					System.out.print(df.format(stats[row][col].getMean())+" ("+df.format(stats[row][col].getStandardDeviation())+")");
				}
				if (col != stats[row].length-1) {
					System.out.print("\t");
				}
			}
			System.out.println();
		}
	}
	
	/** Computes the precision of the sample data against the target data */
	public static double precision(Instances sampleData, Instances targetData) {
		double relevantAndRetrieved = 0;
		// loop through the sample data
		for (int instIdx = 0; instIdx < sampleData.numInstances(); instIdx++) {
			Instance instance = sampleData.instance(instIdx);
			if (WekaUtils.isMember(instance, targetData)) {
				relevantAndRetrieved++;
			}
		}
		// compute the precision
		double precision = relevantAndRetrieved / sampleData.numInstances();
		return precision;
	}
	
	
	/** Computes the recall of the sample data against the target data */
	public static double recall(Instances sampleData, Instances targetData) {
		double relevantAndRetrieved = 0;
		// loop through the sample data
		for (int instIdx = 0; instIdx < sampleData.numInstances(); instIdx++) {
			Instance instance = sampleData.instance(instIdx);
			if (WekaUtils.isMember(instance, targetData)) {
				relevantAndRetrieved++;
			}
		}
		// compute the recall
		double recall = relevantAndRetrieved / targetData.numInstances();
		return recall;
	}
	
	/** Computes the fmeasure of the sample data against the target data */
	public static double fmeasure(Instances sampleData, Instances targetData) {
		double relevantAndRetrieved = 0;
		// loop through the sample data
		for (int instIdx = 0; instIdx < sampleData.numInstances(); instIdx++) {
			Instance instance = sampleData.instance(instIdx);
			if (WekaUtils.isMember(instance, targetData)) {
				relevantAndRetrieved++;
			}
		}
		// shortcut for the sizes of the two sets being the same
		if (sampleData.numInstances() == targetData.numInstances()) {
			// the precision and recall are the same for the same size sets, so their harmonic mean is just the precision (or recall)
			double fmeasure = relevantAndRetrieved / sampleData.numInstances();
			return fmeasure;
		} else {
			// compute the precision
			double precision = relevantAndRetrieved / sampleData.numInstances();
			// compute the recall
			double recall = relevantAndRetrieved / targetData.numInstances();
			// compute the f-measure
			double fmeasure = 2 * precision * recall / (precision + recall);
			return fmeasure;
		}
	}

	
	public static int rank(double targetValue, double[] values) {
		double[] tempValues = new double[values.length];
		System.arraycopy(values,0,tempValues, 0, values.length);
		Arrays.sort(tempValues);
		
		int rank = Arrays.binarySearch(values,targetValue);
		rank = (rank >= 0) ? rank+1 : -rank - 1;
		
		return rank;
	}
	
	
	/** Divides the data into folds.
	 * @param data
	 * @param numFolds
	 * @return An array of instances, each element of which is a fold.
	 */
	public static Instances[] separateIntoFolds(Instances data, int numFolds) {
		int numItems = data.numInstances() / numFolds;
		Instances[] dataFolds = new Instances[numFolds];
		for (int i=0; i<numFolds; i++) {
			dataFolds[i] = new Instances(data,0);
			for (int j=(i*numItems); j<(i*numItems+numItems); j++) {
				dataFolds[i].add(data.instance(j));
			}
		}
		return dataFolds;
	}
	
	

	
	public static ArrayList<Instances> getFoldTrainingSet(Instances[] trainingSubsets, int heldOutFold) {
		ArrayList<Instances> tempTrainingSets = new ArrayList<Instances>();
		for (int i = 0; i < trainingSubsets.length; i++) {
			if (i == heldOutFold)  continue;
			Instances tempTrainingSet = trainingSubsets[i];
			tempTrainingSets.add(tempTrainingSet);
		}
		return tempTrainingSets;
	}
	
	public static RankedSubsets initializeSubsetsFromFile(Instances data, File subsetsFile) {
		
		// error check arguments
		if (data == null) throw new IllegalArgumentException("data cannot be null.");
		if (subsetsFile == null) throw new IllegalArgumentException("subsetsFile cannot be null.");
		
		ArrayList<Instances> subsets = new ArrayList<Instances>();
		ArrayList<Double> subsetValues = new ArrayList<Double>();
		
		// read in the file, and extract instance sets
		try {
			String s;
			BufferedReader in = new BufferedReader(new FileReader(subsetsFile));
			while ((s = in.readLine()) != null) {
				// parse the line
				try {
					
					// ignore comments and blank lines
					if (s.startsWith("%") || s.startsWith("#") ||(s.length() < 2)) continue;
					
					Instances instances = new Instances(data,0);
					instances.setRelationName(subsetsFile.getName());
					
					// parse out the instances
					String[] sections = s.split("\t");
					String[] strIds = sections[0].split(",");
					
					double value = (sections.length > 1) ? Double.parseDouble(sections[1]) : 1.0;
					subsetValues.add(value);
					
					// loop through each song id, and extract the instance
					for (String strId : strIds) {
						int id = Integer.parseInt(strId);
						// loop through all data, find and add the instance with the correct id
						int numInstances = data.numInstances();
						boolean found = false;
						for (int instIdx=0; instIdx<numInstances; instIdx++) {
							Instance inst = data.instance(instIdx);
							if (inst.value(0) ==  id) {
								instances.add(inst);
								found = true;
								break;
							}
						}
						if (!found) {
							System.err.println("No instance with id "+id+".");
						}
					}
					
					// store the playlist
					subsets.add(instances);
					
					
				} catch (Exception e) {
					System.err.println("Skipping improperly formatted line: "+s);
					System.err.println(e.getMessage());
				}
				
			}
			
		} catch (Exception e) {
			System.err.println("File I/O Error.");
			e.printStackTrace();
		}
		
		RankedSubsets rankedSubsets = new RankedSubsets();
		rankedSubsets.subsets = (Instances[]) subsets.toArray(new Instances[0]);
		Double[] values = (Double[]) subsetValues.toArray(new Double[0]);
		rankedSubsets.subsetValues = new double[values.length];
		int valueIdx = 0;
		for (double value : values) {
			rankedSubsets.subsetValues[valueIdx++] = value;
		}
		return rankedSubsets;
	}
	
	
	/** Checks that a value is within the specified range.
	 * @param value The value to check.
	 * @param minValue The minimum value allowed.
	 * @param maxValue The maximum value allowed.
	 * @return <code>true</code> if minValue <= value <= maxValue, <code>false</code> otherwise.
	 */
	public static boolean valueInRange(double value, double minValue, double maxValue) {
		double EPSILON = 1.0E-10;
		return (value >= (minValue - EPSILON) && value <= (maxValue + EPSILON));
	}
	
	public static double[] parseDoubleArray(String strDoubles) {
		String[] strDoublesArray = strDoubles.split(",");
		double[] doubles = new double[strDoublesArray.length];
		for (int i=0; i<doubles.length; i++) {
			doubles[i] = Double.parseDouble(strDoublesArray[i]);
		}
		return doubles;
	}
	
	public static int[] parseIntegerArray(String strInts) {
		String[] strIntsArray = strInts.split(",");
		int[] ints = new int[strIntsArray.length];
		for (int i=0; i<ints.length; i++) {
			ints[i] = Integer.parseInt(strIntsArray[i]);
		}
		return ints;
	}
}

